Package: kernlab
Version: 0.9-33
Title: Kernel-Based Machine Learning Lab
Authors@R: c(person("Alexandros", "Karatzoglou", role = c("aut", "cre"),
                    email = "alexandros.karatzoglou@gmail.com"),
             person("Alex", "Smola", role = "aut"),
             person("Kurt", "Hornik", role = "aut",
	            email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911")),
	     person("National ICT Australia (NICTA)",
	            role = "cph"),
	     person(c("Michael", "A."), "Maniscalco",
	            role = c("ctb", "cph")),
             person(c("Choon", "Hui"), "Teo", role = "ctb"))
Description: Kernel-based machine learning methods for classification,
        regression, clustering, novelty detection, quantile regression
        and dimensionality reduction.  Among other methods 'kernlab'
        includes Support Vector Machines, Spectral Clustering, Kernel
        PCA, Gaussian Processes and a QP solver.
Depends: R (>= 2.10)
Imports: methods, stats, grDevices, graphics
LazyLoad: Yes
License: GPL-2
NeedsCompilation: yes
Packaged: 2024-08-13 14:40:27 UTC; hornik
Author: Alexandros Karatzoglou [aut, cre],
  Alex Smola [aut],
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  National ICT Australia (NICTA) [cph],
  Michael A. Maniscalco [ctb, cph],
  Choon Hui Teo [ctb]
Maintainer: Alexandros Karatzoglou <alexandros.karatzoglou@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-13 15:25:01 UTC
