useDynLib("kernlab", .registration = TRUE)

import("methods")
importFrom("stats", "coef", "delete.response", "fitted", "kmeans",
           "median", "model.extract", "model.matrix", "na.action",
           "na.omit", "predict", "quantile", "rnorm", "runif", "sd",
           "terms", "var") 
importFrom("graphics", "axis", "filled.contour", "plot", "points", "title")
importFrom("grDevices", "hcl")

export(	
       ## kernel functions
       "rbfdot",
       "laplacedot",
       "besseldot",
       "polydot",
       "tanhdot",
       "vanilladot",
       "anovadot",
       "splinedot",
       "stringdot",
       "kernelMatrix",
       "kernelMult",
       "kernelPol",
       "kernelFast",
       "as.kernelMatrix",
	
       ## High level functions 
       "kmmd",
       "kpca",
       "kcca",
       "kha",
       "specc",
       "kkmeans",
       "ksvm",
       "rvm",
       "gausspr",
       "ranking",
       "csi",
       "lssvm",
       "kqr",
       
       ## Utility functions
       "ipop",
       "inchol",
       "couple",
       "sigest",
       
       ## Accessor functions
       
       ## VM
       "type",
       "prior",
       "alpha",
       "alphaindex",
       "kernelf",
       "kpar",
       "param",
       "scaling",
       "xmatrix",
       "ymatrix",
       "lev",
       "kcall",
       "error",
       "cross",
       "SVindex",
	"nSV",
       "RVindex",
       "prob.model",
       "b",
       "obj",

       ## kpca
       "rotated",
       "eig",
       "pcv",
       
       ## ipop
       "primal",
       "dual",
       "how",
       
       ## kcca
       "kcor",
       "xcoef",
       "ycoef",
      ## "xvar",
      ## "yvar",
       
       ## specc
       "size",
       "centers",
       "withinss",
       
       ## rvm
       "mlike",
       "nvar",
       
       ## ranking
       "convergence",
       "edgegraph",
       
       ## onlearn
       "onlearn",
       "inlearn",
       "buffer",
       "rho",

       ## kfa
       "kfa",
	
       ## inc.chol
       "pivots",
       "diagresidues",
       "maxresiduals",
       
       ## csi
       "R",
       "Q",
       "truegain",
       "predgain",
       
       ## kmmd
       "H0",
       "AsympH0",
       "Radbound",
       "Asymbound",
       "mmdstats"
       )

exportMethods("coef", "fitted", "plot", "predict", "show")

exportClasses("ksvm", "kmmd", "rvm", "ipop", "gausspr", "lssvm", "kpca", "kha",
              "kcca", "kernel", "rbfkernel", "laplacekernel",
              "besselkernel", "tanhkernel", "polykernel","fourierkernel",
              "vanillakernel", "anovakernel", "splinekernel",
              "stringkernel", "specc", "ranking", "inchol", "onlearn",
              "kfa", "csi","kqr",
              "kernelMatrix","kfunction")

